@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -36,7 +36,7 @@
 #endif  /* HAVE_MEMORY_H */
 
 #include "allocator.h"
-
+#include "malloc_stub.h"
 
 /* create allocator object */
 SIXELSTATUS
@@ -196,20 +196,30 @@ sixel_allocator_free(
 
 
 #if HAVE_TESTS
+volatile int sixel_debug_malloc_counter;
+
 void *
-sixel_bad_malloc(size_t n)
+sixel_bad_malloc(size_t size)
 {
-    (void) n;
+    return sixel_debug_malloc_counter-- == 0 ? NULL: malloc(size);
+}
+
+
+void *
+sixel_bad_calloc(size_t count, size_t size)
+{
+    (void) count;
+    (void) size;
 
     return NULL;
 }
 
 
 void *
-sixel_bad_realloc(void *p, size_t n)
+sixel_bad_realloc(void *ptr, size_t size)
 {
-    (void) p;
-    (void) n;
+    (void) ptr;
+    (void) size;
 
     return NULL;
 }
@@ -276,6 +286,27 @@ error:
 }
 
 
+static int
+test2(void)
+{
+    int nret = EXIT_FAILURE;
+    SIXELSTATUS status;
+    sixel_allocator_t *allocator = NULL;
+
+    sixel_debug_malloc_counter = 1;
+
+    status = sixel_allocator_new(&allocator, sixel_bad_malloc, calloc, realloc, free);
+    if (status == SIXEL_BAD_ALLOCATION) {
+        goto error;
+    }
+
+    nret = EXIT_SUCCESS;
+
+error:
+    return nret;
+}
+
+
 int
 sixel_allocator_tests_main(void)
 {
@@ -285,6 +316,7 @@ sixel_allocator_tests_main(void)
 
     static testcase const testcases[] = {
         test1,
+        test2
     };
 
     for (i = 0; i < sizeof(testcases) / sizeof(testcase); ++i) {
@@ -301,7 +333,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-/* Hello emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
