@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -170,7 +170,14 @@ wait_file(int fd, int usec)
     tv.tv_sec = usec / 1000000;
     tv.tv_usec = usec % 1000000;
     FD_ZERO(&rfds);
+#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
     FD_SET(fd, &rfds);
+#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+# pragma GCC diagnostic pop
+#endif
     ret = select(fd + 1, &rfds, NULL, NULL, &tv);
 #else
     (void) fd;
@@ -250,7 +257,7 @@ sixel_chunk_from_file(
     SIXELSTATUS status = SIXEL_FALSE;
     int ret;
     FILE *f;
-    int n;
+    size_t n;
     size_t const bucket_size = 4096;
 
     status = open_binary_file(&f, filename);
@@ -291,7 +298,7 @@ sixel_chunk_from_file(
             }
         }
         n = fread(pchunk->buffer + pchunk->size, 1, 4096, f);
-        if (n <= 0) {
+        if (n == 0) {
             break;
         }
         pchunk->size += n;
@@ -505,7 +512,7 @@ static int
 test2(void)
 {
     int nret = EXIT_FAILURE;
-    sixel_chunk_t *chunk;
+    sixel_chunk_t *chunk = NULL;
     SIXELSTATUS status = SIXEL_FALSE;
 
     status = sixel_chunk_new(&chunk, NULL, 0, NULL, NULL);
@@ -525,6 +532,59 @@ error:
 }
 
 
+static int
+test3(void)
+{
+    int nret = EXIT_FAILURE;
+    sixel_chunk_t *chunk;
+    sixel_allocator_t *allocator = NULL;
+    SIXELSTATUS status = SIXEL_FALSE;
+
+    sixel_debug_malloc_counter = 1;
+
+    status = sixel_allocator_new(&allocator, sixel_bad_malloc, NULL, NULL, NULL);
+    if (SIXEL_FAILED(status)) {
+        goto error;
+    }
+
+    status = sixel_chunk_new(&chunk, "../images/map8.six", 0, NULL, allocator);
+    if (status != SIXEL_BAD_ALLOCATION) {
+        goto error;
+    }
+
+    nret = EXIT_SUCCESS;
+
+error:
+    return nret;
+}
+
+
+static int
+test4(void)
+{
+    int nret = EXIT_FAILURE;
+    sixel_chunk_t *chunk;
+    sixel_allocator_t *allocator = NULL;
+    SIXELSTATUS status = SIXEL_FALSE;
+
+    sixel_debug_malloc_counter = 2;
+
+    status = sixel_allocator_new(&allocator, sixel_bad_malloc, NULL, NULL, NULL);
+    if (SIXEL_FAILED(status)) {
+        goto error;
+    }
+    status = sixel_chunk_new(&chunk, "../images/map8.six", 0, NULL, allocator);
+    if (status != SIXEL_BAD_ALLOCATION) {
+        goto error;
+    }
+
+    nret = EXIT_SUCCESS;
+
+error:
+    return nret;
+}
+
+
 int
 sixel_chunk_tests_main(void)
 {
@@ -535,6 +595,8 @@ sixel_chunk_tests_main(void)
     static testcase const testcases[] = {
         test1,
         test2,
+        test3,
+        test4,
     };
 
     for (i = 0; i < sizeof(testcases) / sizeof(testcase); ++i) {
@@ -551,7 +613,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
