@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -257,9 +257,9 @@ sixel_dither_new(
                                                default allocator */
 {
     SIXELSTATUS status = SIXEL_FALSE;
-    int headsize;
-    int datasize;
-    int wholesize;
+    size_t headsize;
+    size_t datasize;
+    size_t wholesize;
     int quality_mode;
 
     if (ppdither == NULL) {
@@ -291,7 +291,7 @@ sixel_dither_new(
         quality_mode = SIXEL_QUALITY_LOW;
     }
     headsize = sizeof(sixel_dither_t);
-    datasize = ncolors * 3;
+    datasize = (size_t)(ncolors * 3);
     wholesize = headsize + datasize;
 
     *ppdither = (sixel_dither_t *)sixel_allocator_malloc(allocator, wholesize);
@@ -530,11 +530,14 @@ sixel_dither_initialize(
 
     sixel_dither_set_pixelformat(dither, pixelformat);
 
-    if (pixelformat != SIXEL_PIXELFORMAT_RGB888) {
-
+    switch (pixelformat) {
+    case SIXEL_PIXELFORMAT_RGB888:
+        input_pixels = data;
+        break;
+    default:
         /* normalize pixelformat */
         normalized_pixels
-            = (unsigned char *)sixel_allocator_malloc(dither->allocator, width * height * 3);
+            = (unsigned char *)sixel_allocator_malloc(dither->allocator, (size_t)(width * height * 3));
         if (normalized_pixels == NULL) {
             sixel_helper_set_additional_message(
                 "sixel_dither_initialize: sixel_allocator_malloc() failed.");
@@ -553,8 +556,7 @@ sixel_dither_initialize(
             goto end;
         }
         input_pixels = normalized_pixels;
-    } else {
-        input_pixels = data;
+        break;
     }
 
     sixel_dither_set_method_for_largest(dither, method_for_largest);
@@ -563,11 +565,11 @@ sixel_dither_initialize(
 
     status = sixel_quant_make_palette(&buf,
                                       input_pixels,
-                                      width * height * 3,
+                                      (unsigned int)(width * height * 3),
                                       SIXEL_PIXELFORMAT_RGB888,
-                                      dither->reqcolors,
-                                      &dither->ncolors,
-                                      &dither->origcolors,
+                                      (unsigned int)dither->reqcolors,
+                                      (unsigned int *)&dither->ncolors,
+                                      (unsigned int *)&dither->origcolors,
                                       dither->method_for_largest,
                                       dither->method_for_rep,
                                       dither->quality_mode,
@@ -575,7 +577,7 @@ sixel_dither_initialize(
     if (SIXEL_FAILED(status)) {
         goto end;
     }
-    memcpy(dither->palette, buf, dither->ncolors * 3);
+    memcpy(dither->palette, buf, (size_t)(dither->ncolors * 3));
 
     dither->optimized = 1;
     if (dither->origcolors <= dither->ncolors) {
@@ -592,6 +594,7 @@ end:
 }
 
 
+/* set diffusion type, choose from enum methodForDiffuse */
 SIXELAPI void
 sixel_dither_set_diffusion_type(
     sixel_dither_t  /* in */ *dither,
@@ -608,6 +611,7 @@ sixel_dither_set_diffusion_type(
 }
 
 
+/* get number of palette colors */
 SIXELAPI int
 sixel_dither_get_num_of_palette_colors(
     sixel_dither_t  /* in */ *dither)
@@ -634,6 +638,7 @@ sixel_dither_get_num_of_histgram_colors(
 }
 
 
+/* get palette */
 SIXELAPI unsigned char *
 sixel_dither_get_palette(
     sixel_dither_t /* in */ *dither)  /* dither context object */
@@ -648,10 +653,11 @@ sixel_dither_set_palette(
     sixel_dither_t /* in */ *dither,   /* dither context object */
     unsigned char  /* in */ *palette)
 {
-    memcpy(dither->palette, palette, dither->ncolors * 3);
+    memcpy(dither->palette, palette, (size_t)(dither->ncolors * 3));
 }
 
 
+/* set the factor of complexion color correcting */
 SIXELAPI void
 sixel_dither_set_complexion_score(
     sixel_dither_t /* in */ *dither,  /* dither context object */
@@ -661,6 +667,7 @@ sixel_dither_set_complexion_score(
 }
 
 
+/* set whether omitting palette difinition */
 SIXELAPI void
 sixel_dither_set_body_only(
     sixel_dither_t /* in */ *dither,     /* dither context object */
@@ -671,6 +678,7 @@ sixel_dither_set_body_only(
 }
 
 
+/* set whether optimize palette size */
 SIXELAPI void
 sixel_dither_set_optimize_palette(
     sixel_dither_t /* in */ *dither,   /* dither context object */
@@ -681,6 +689,7 @@ sixel_dither_set_optimize_palette(
 }
 
 
+/* set pixelformat */
 SIXELAPI void
 sixel_dither_set_pixelformat(
     sixel_dither_t /* in */ *dither,     /* dither context object */
@@ -690,6 +699,7 @@ sixel_dither_set_pixelformat(
 }
 
 
+/* set transparent */
 SIXELAPI void
 sixel_dither_set_transparent(
     sixel_dither_t /* in */ *dither,      /* dither context object */
@@ -699,6 +709,7 @@ sixel_dither_set_transparent(
 }
 
 
+/* set transparent */
 SIXELAPI unsigned char *
 sixel_dither_apply_palette(
     sixel_dither_t  /* in */ *dither,
@@ -707,7 +718,7 @@ sixel_dither_apply_palette(
     int             /* in */ height)
 {
     SIXELSTATUS status = SIXEL_FALSE;
-    int bufsize;
+    size_t bufsize;
     unsigned char *dest = NULL;
     int ncolors;
     unsigned char *normalized_pixels = NULL;
@@ -722,7 +733,7 @@ sixel_dither_apply_palette(
 
     sixel_dither_ref(dither);
 
-    bufsize = width * height * sizeof(unsigned char);
+    bufsize = (size_t)(width * height) * sizeof(unsigned char);
     dest = (unsigned char *)sixel_allocator_malloc(dither->allocator, bufsize);
     if (dest == NULL) {
         sixel_helper_set_additional_message(
@@ -739,7 +750,7 @@ sixel_dither_apply_palette(
     if (dither->cachetable == NULL && dither->optimized) {
         if (dither->palette != pal_mono_dark && dither->palette != pal_mono_light) {
             dither->cachetable = (unsigned short *)sixel_allocator_calloc(dither->allocator,
-                                                                          1 << 3 * 5,
+                                                                          (size_t)(1 << 3 * 5),
                                                                           sizeof(unsigned short));
             if (dither->cachetable == NULL) {
                 sixel_helper_set_additional_message(
@@ -753,7 +764,7 @@ sixel_dither_apply_palette(
     if (dither->pixelformat != SIXEL_PIXELFORMAT_RGB888) {
         /* normalize pixelformat */
         normalized_pixels
-            = (unsigned char *)sixel_allocator_malloc(dither->allocator, width * height * 3);
+            = (unsigned char *)sixel_allocator_malloc(dither->allocator, (size_t)(width * height * 3));
         if (normalized_pixels == NULL) {
             sixel_helper_set_additional_message(
                 "sixel_dither_new: sixel_allocator_malloc() failed.");
@@ -883,6 +894,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
