@@ -5,7 +5,7 @@
  * Hayaki Saito <saitoha@me.com> modified this and re-licensed
  * it under the MIT license.
  *
- * Copyright (c) 2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -178,10 +178,10 @@ gif_init_frame(
     frame->delay = pg->delay;
     ncolors = 2 << (pg->flags & 7);
     if (frame->palette == NULL) {
-        frame->palette = (unsigned char *)sixel_allocator_malloc(frame->allocator, ncolors * 3);
+        frame->palette = (unsigned char *)sixel_allocator_malloc(frame->allocator, (size_t)(ncolors * 3));
     } else if (frame->ncolors < ncolors) {
         sixel_allocator_free(frame->allocator, frame->palette);
-        frame->palette = (unsigned char *)sixel_allocator_malloc(frame->allocator, ncolors * 3);
+        frame->palette = (unsigned char *)sixel_allocator_malloc(frame->allocator, (size_t)(ncolors * 3));
     }
     if (frame->palette == NULL) {
         sixel_helper_set_additional_message(
@@ -194,14 +194,14 @@ gif_init_frame(
         frame->pixelformat = SIXEL_PIXELFORMAT_PAL8;
         sixel_allocator_free(frame->allocator, frame->pixels);
         frame->pixels = (unsigned char *)sixel_allocator_malloc(frame->allocator,
-                                                                frame->width * frame->height);
+                                                                (size_t)(frame->width * frame->height));
         if (frame->pixels == NULL) {
             sixel_helper_set_additional_message(
                 "sixel_allocator_malloc() failed in gif_init_frame().");
             status = SIXEL_BAD_ALLOCATION;
             goto end;
         }
-        memcpy(frame->pixels, pg->out, frame->width * frame->height);
+        memcpy(frame->pixels, pg->out, (size_t)(frame->width * frame->height));
 
         for (i = 0; i < frame->ncolors; ++i) {
             frame->palette[i * 3 + 0] = pg->color_table[i * 3 + 2];
@@ -232,7 +232,7 @@ gif_init_frame(
     } else {
         frame->pixelformat = SIXEL_PIXELFORMAT_RGB888;
         frame->pixels = (unsigned char *)sixel_allocator_malloc(frame->allocator,
-                                                                pg->w * pg->h * 3);
+                                                                (size_t)(pg->w * pg->h * 3));
         if (frame->pixels == NULL) {
             sixel_helper_set_additional_message(
                 "sixel_allocator_malloc() failed in gif_init_frame().");
@@ -263,7 +263,7 @@ gif_out_code(
     /* recurse to decode the prefixes, since the linked-list is backwards,
        and working backwards through an interleaved image would be nasty */
     if (g->codes[code].prefix >= 0) {
-        gif_out_code(g, g->codes[code].prefix);
+        gif_out_code(g, (unsigned short)g->codes[code].prefix);
     }
 
     if (g->cur_y >= g->max_y) {
@@ -492,7 +492,7 @@ gif_load_next(
                     status = SIXEL_RUNTIME_ERROR;
                     goto end;
                 }
-                memcpy(buffer, s->img_buffer, len);
+                memcpy(buffer, s->img_buffer, (size_t)len);
                 s->img_buffer += len;
                 buffer[len] = 0;
                 if (len == 11 && strcmp((char *)buffer, "NETSCAPE2.0") == 0) {
@@ -505,6 +505,9 @@ gif_load_next(
                         case 0x01:
                             g->loop_count = gif_get16le(s);
                             break;
+                        default:
+                            g->loop_count = 1;
+                            break;
                         }
                     }
                 }
@@ -536,6 +539,10 @@ end:
     return status;
 }
 
+typedef union _fn_pointer {
+    sixel_load_image_function fn;
+    void *                    p;
+} fn_pointer;
 
 SIXELSTATUS
 load_gif(
@@ -554,7 +561,9 @@ load_gif(
     gif_t g;
     SIXELSTATUS status = SIXEL_FALSE;
     sixel_frame_t *frame;
+    fn_pointer fnp;
 
+    fnp.p = fn_load;
     g.out = NULL;
 
     status = sixel_frame_new(&frame, allocator);
@@ -568,9 +577,7 @@ load_gif(
     if (status != SIXEL_OK) {
         goto end;
     }
-    frame->width = g.w,
-    frame->height = g.h,
-    g.out = (unsigned char *)sixel_allocator_malloc(allocator, g.w * g.h);
+    g.out = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)(g.w * g.h));
     if (g.out == NULL) {
         sixel_helper_set_additional_message(
             "load_gif: sixel_allocator_malloc() failed.");
@@ -601,18 +608,20 @@ load_gif(
                 break;
             }
 
+            frame->width = g.w;
+            frame->height = g.h;
             status = gif_init_frame(frame, &g, bgcolor, reqcolors, fuse_palette);
             if (status != SIXEL_OK) {
                 goto end;
             }
 
-            status = ((sixel_load_image_function)fn_load)(frame, context);
+            status = fnp.fn(frame, context);
             if (status != SIXEL_OK) {
                 goto end;
             }
 
             if (fstatic) {
-                break;
+                goto end;
             }
             ++frame->frame_no;
         }
@@ -633,8 +642,8 @@ load_gif(
     }
 
 end:
-    sixel_frame_unref(frame);
     sixel_allocator_free(frame->allocator, g.out);
+    sixel_frame_unref(frame);
 
     return status;
 }
@@ -677,7 +686,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
