@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -37,10 +37,6 @@
 # define memcpy(d, s, n) (bcopy ((s), (d), (n)))
 #endif
 
-#if !defined(HAVE_MEMMOVE)
-# define memmove(d, s, n) (bcopy ((s), (d), (n)))
-#endif
-
 #ifdef HAVE_GDK_PIXBUF2
 # if HAVE_DIAGNOSTIC_TYPEDEF_REDEFINITION
 #   pragma GCC diagnostic push
@@ -89,7 +85,7 @@ stbi_realloc(void *p, size_t n)
 void
 stbi_free(void *p)
 {
-    return sixel_allocator_free(stbi_allocator, p);
+    sixel_allocator_free(stbi_allocator, p);
 }
 
 #define STBI_MALLOC stbi_malloc
@@ -98,7 +94,45 @@ stbi_free(void *p)
 
 #define STBI_NO_STDIO 1
 #define STB_IMAGE_IMPLEMENTATION 1
+#define STBI_NO_GIF
+#define STBI_NO_PNM
+
+#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
+#if HAVE_DIAGNOSTIC_STRICT_OVERFLOW
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wstrict-overflow"
+#endif
+#if HAVE_DIAGNOSTIC_SWITCH_DEFAULT
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wswitch-default"
+#endif
+#if HAVE_DIAGNOSTIC_SHADOW
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wshadow"
+#endif
+#if HAVE_DIAGNOSTIC_DOUBLE_PROMOTION
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdouble-promotion"
+#endif
 #include "stb_image.h"
+#if HAVE_DIAGNOSTIC_DOUBLE_PROMOTION
+# pragma GCC diagnostic pop
+#endif
+#if HAVE_DIAGNOSTIC_SHADOW
+# pragma GCC diagnostic pop
+#endif
+#if HAVE_DIAGNOSTIC_SWITCH_DEFAULT
+# pragma GCC diagnostic pop
+#endif
+#if HAVE_DIAGNOSTIC_STRICT_OVERFLOW
+# pragma GCC diagnostic pop
+#endif
+#if HAVE_DIAGNOSTIC_SIGN_CONVERSION
+# pragma GCC diagnostic pop
+#endif
 
 
 # if HAVE_JPEG
@@ -106,14 +140,15 @@ stbi_free(void *p)
 static SIXELSTATUS
 load_jpeg(unsigned char **result,
           unsigned char *data,
-          int datasize,
+          size_t datasize,
           int *pwidth,
           int *pheight,
           int *ppixelformat,
           sixel_allocator_t *allocator)
 {
     SIXELSTATUS status = SIXEL_JPEG_ERROR;
-    int row_stride, size;
+    JDIMENSION row_stride;
+    size_t size;
     JSAMPARRAY buffer;
     struct jpeg_decompress_struct cinfo;
     struct jpeg_error_mgr pub;
@@ -137,10 +172,10 @@ load_jpeg(unsigned char **result,
     }
 
     *ppixelformat = SIXEL_PIXELFORMAT_RGB888;
-    *pwidth = cinfo.output_width;
-    *pheight = cinfo.output_height;
+    *pwidth = (int)cinfo.output_width;
+    *pheight = (int)cinfo.output_height;
 
-    size = *pwidth * *pheight * cinfo.output_components;
+    size = (size_t)(*pwidth * *pheight * cinfo.output_components);
     *result = (unsigned char *)sixel_allocator_malloc(allocator, size);
     if (*result == NULL) {
         sixel_helper_set_additional_message(
@@ -148,7 +183,7 @@ load_jpeg(unsigned char **result,
         status = SIXEL_BAD_ALLOCATION;
         goto end;
     }
-    row_stride = cinfo.output_width * cinfo.output_components;
+    row_stride = cinfo.output_width * (unsigned int)cinfo.output_components;
     buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo, JPOOL_IMAGE, row_stride, 1);
 
     while (cinfo.output_scanline < cinfo.output_height) {
@@ -224,7 +259,7 @@ read_palette(png_structp png_ptr,
 static SIXELSTATUS
 load_png(unsigned char      /* out */ **result,
          unsigned char      /* in */  *buffer,
-         int                /* in */  size,
+         size_t             /* in */  size,
          int                /* out */ *psx,
          int                /* out */ *psy,
          unsigned char      /* out */ **ppalette,
@@ -268,8 +303,8 @@ load_png(unsigned char      /* out */ **result,
 
     png_set_read_fn(png_ptr,(png_voidp)&read_chunk, read_png);
     png_read_info(png_ptr, info_ptr);
-    *psx = png_get_image_width(png_ptr, info_ptr);
-    *psy = png_get_image_height(png_ptr, info_ptr);
+    *psx = (int)png_get_image_width(png_ptr, info_ptr);
+    *psy = (int)png_get_image_height(png_ptr, info_ptr);
     bitdepth = png_get_bit_depth(png_ptr, info_ptr);
     if (bitdepth == 16) {
 #  if HAVE_DEBUG
@@ -325,7 +360,7 @@ load_png(unsigned char      /* out */ **result,
         } else {
             switch (bitdepth) {
             case 1:
-                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, *pncolors * 3);
+                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)*pncolors * 3);
                 if (*ppalette == NULL) {
                     sixel_helper_set_additional_message(
                         "load_png: sixel_allocator_malloc() failed.");
@@ -337,7 +372,7 @@ load_png(unsigned char      /* out */ **result,
                 *pixelformat = SIXEL_PIXELFORMAT_PAL1;
                 break;
             case 2:
-                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, *pncolors * 3);
+                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)*pncolors * 3);
                 if (*ppalette == NULL) {
                     sixel_helper_set_additional_message(
                         "load_png: sixel_allocator_malloc() failed.");
@@ -349,7 +384,7 @@ load_png(unsigned char      /* out */ **result,
                 *pixelformat = SIXEL_PIXELFORMAT_PAL2;
                 break;
             case 4:
-                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, *pncolors * 3);
+                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)*pncolors * 3);
                 if (*ppalette == NULL) {
                     sixel_helper_set_additional_message(
                         "load_png: sixel_allocator_malloc() failed.");
@@ -361,7 +396,7 @@ load_png(unsigned char      /* out */ **result,
                 *pixelformat = SIXEL_PIXELFORMAT_PAL4;
                 break;
             case 8:
-                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, *pncolors * 3);
+                *ppalette = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)*pncolors * 3);
                 if (*ppalette == NULL) {
                     sixel_helper_set_additional_message(
                         "load_png: sixel_allocator_malloc() failed.");
@@ -492,14 +527,14 @@ load_png(unsigned char      /* out */ **result,
         goto cleanup;
     }
     depth = sixel_helper_compute_depth(*pixelformat);
-    *result = (unsigned char *)sixel_allocator_malloc(allocator, *psx * *psy * depth);
+    *result = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)(*psx * *psy * depth));
     if (*result == NULL) {
         sixel_helper_set_additional_message(
             "load_png: sixel_allocator_malloc() failed.");
         status = SIXEL_BAD_ALLOCATION;
         goto cleanup;
     }
-    rows = (unsigned char **)sixel_allocator_malloc(allocator, *psy * sizeof(unsigned char *));
+    rows = (unsigned char **)sixel_allocator_malloc(allocator, (size_t)*psy * sizeof(unsigned char *));
     if (rows == NULL) {
         sixel_helper_set_additional_message(
             "load_png: sixel_allocator_malloc() failed.");
@@ -511,7 +546,7 @@ load_png(unsigned char      /* out */ **result,
     case SIXEL_PIXELFORMAT_PAL2:
     case SIXEL_PIXELFORMAT_PAL4:
         for (i = 0; i < *psy; ++i) {
-            rows[i] = *result + (depth * *psx * bitdepth + 7) / 8 * i;
+            rows[i] = *result + (depth * *psx * (int)bitdepth + 7) / 8 * i;
         }
         break;
     default:
@@ -568,7 +603,7 @@ load_sixel(unsigned char        /* out */ **result,
     }
     if (ppalette == NULL || colors > reqcolors) {
         *ppixelformat = SIXEL_PIXELFORMAT_RGB888;
-        *result = (unsigned char *)sixel_allocator_malloc(allocator, *psx * *psy * 3);
+        *result = (unsigned char *)sixel_allocator_malloc(allocator, (size_t)(*psx * *psy * 3));
         if (*result == NULL) {
             sixel_helper_set_additional_message(
                 "load_sixel: sixel_allocator_malloc() failed.");
@@ -695,8 +730,12 @@ chunk_is_jpeg(sixel_chunk_t const *chunk)
 }
 #endif  /* HAVE_JPEG */
 
+typedef union _fn_pointer {
+    sixel_load_image_function fn;
+    void *                    p;
+} fn_pointer;
 
-
+/* load images using builtin image loaders */
 static SIXELSTATUS
 load_with_builtin(
     sixel_chunk_t const       /* in */     *pchunk,      /* image data */
@@ -713,6 +752,7 @@ load_with_builtin(
     sixel_frame_t *frame = NULL;
     char message[256];
     int nwrite;
+    fn_pointer fnp;
 
     if (chunk_is_sixel(pchunk)) {
         status = sixel_frame_new(&frame, pchunk->allocator);
@@ -721,7 +761,7 @@ load_with_builtin(
         }
         status = load_sixel(&frame->pixels,
                             pchunk->buffer,
-                            pchunk->size,
+                            (int)pchunk->size,
                             &frame->width,
                             &frame->height,
                             fuse_palette ? &frame->palette: NULL,
@@ -739,7 +779,7 @@ load_with_builtin(
         }
         /* pnm */
         status = load_pnm(pchunk->buffer,
-                          pchunk->size,
+                          (int)pchunk->size,
                           frame->allocator,
                           &frame->pixels,
                           &frame->width,
@@ -794,14 +834,15 @@ load_with_builtin(
     }
 #endif  /* HAVE_LIBPNG */
     else if (chunk_is_gif(pchunk)) {
+        fnp.fn = fn_load;
         status = load_gif(pchunk->buffer,
-                          pchunk->size,
+                          (int)pchunk->size,
                           bgcolor,
                           reqcolors,
                           fuse_palette,
                           fstatic,
                           loop_control,
-                          (void *)fn_load,
+                          fnp.p,
                           context,
                           pchunk->allocator);
         if (SIXEL_FAILED(status)) {
@@ -817,7 +858,7 @@ load_with_builtin(
             goto end;
         }
         stbi_allocator = pchunk->allocator;
-        stbi__start_mem(&s, pchunk->buffer, pchunk->size);
+        stbi__start_mem(&s, pchunk->buffer, (int)pchunk->size);
         frame->pixels = stbi__load_main(&s, &frame->width, &frame->height, &depth, 3);
         if (!frame->pixels) {
             sixel_helper_set_additional_message(stbi_failure_reason());
@@ -882,7 +923,7 @@ load_with_gdkpixbuf(
     GdkPixbufLoader *loader = NULL;
 #if 1
     GdkPixbufAnimationIter *it;
-    GTimeVal time;
+    GTimeVal time_val;
 #endif
     sixel_frame_t *frame = NULL;
     int stride;
@@ -902,7 +943,7 @@ load_with_gdkpixbuf(
 #if (!GLIB_CHECK_VERSION(2, 36, 0))
     g_type_init();
 #endif
-    g_get_current_time(&time);
+    g_get_current_time(&time_val);
     loader = gdk_pixbuf_loader_new();
     gdk_pixbuf_loader_write(loader, pchunk->buffer, pchunk->size, NULL);
     animation = gdk_pixbuf_loader_get_animation(loader);
@@ -915,7 +956,7 @@ load_with_gdkpixbuf(
         frame->width = gdk_pixbuf_get_width(pixbuf);
         frame->height = gdk_pixbuf_get_height(pixbuf);
         stride = gdk_pixbuf_get_rowstride(pixbuf);
-        frame->pixels = sixel_allocator_malloc(pchunk->allocator, frame->height * stride);
+        frame->pixels = sixel_allocator_malloc(pchunk->allocator, (size_t)(frame->height * stride));
         if (frame->pixels == NULL) {
             sixel_helper_set_additional_message(
                 "load_with_gdkpixbuf: sixel_allocator_malloc() failed.");
@@ -932,12 +973,12 @@ load_with_gdkpixbuf(
         p = gdk_pixbuf_get_pixels(pixbuf);
         if (stride == frame->width * depth) {
             memcpy(frame->pixels, gdk_pixbuf_get_pixels(pixbuf),
-                   frame->height * stride);
+                   (size_t)(frame->height * stride));
         } else {
             for (i = 0; i < frame->height; ++i) {
                 memcpy(frame->pixels + frame->width * depth * i,
                        p + stride * i,
-                       frame->width * depth);
+                       (size_t)(frame->width * depth));
             }
         }
         status = fn_load(frame, context);
@@ -945,15 +986,15 @@ load_with_gdkpixbuf(
             goto end;
         }
     } else {
-        g_get_current_time(&time);
+        g_get_current_time(&time_val);
 
         frame->frame_no = 0;
 
-        it = gdk_pixbuf_animation_get_iter(animation, &time);
+        it = gdk_pixbuf_animation_get_iter(animation, &time_val);
         for (;;) {
             while (!gdk_pixbuf_animation_iter_on_currently_loading_frame(it)) {
                 frame->delay = gdk_pixbuf_animation_iter_get_delay_time(it);
-                g_time_val_add(&time, frame->delay * 1000);
+                g_time_val_add(&time_val, frame->delay * 1000);
                 frame->delay /= 10;
                 pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(it);
                 if (pixbuf == NULL) {
@@ -962,7 +1003,7 @@ load_with_gdkpixbuf(
                 frame->width = gdk_pixbuf_get_width(pixbuf);
                 frame->height = gdk_pixbuf_get_height(pixbuf);
                 stride = gdk_pixbuf_get_rowstride(pixbuf);
-                frame->pixels = sixel_allocator_malloc(pchunk->allocator, frame->height * stride);
+                frame->pixels = sixel_allocator_malloc(pchunk->allocator, (size_t)(frame->height * stride));
                 if (frame->pixels == NULL) {
                     sixel_helper_set_additional_message(
                         "load_with_gdkpixbuf: sixel_allocator_malloc() failed.");
@@ -979,16 +1020,16 @@ load_with_gdkpixbuf(
                 p = gdk_pixbuf_get_pixels(pixbuf);
                 if (stride == frame->width * depth) {
                     memcpy(frame->pixels, gdk_pixbuf_get_pixels(pixbuf),
-                           frame->height * stride);
+                           (size_t)(frame->height * stride));
                 } else {
                     for (i = 0; i < frame->height; ++i) {
                         memcpy(frame->pixels + frame->width * depth * i,
                                p + stride * i,
-                               frame->width * depth);
+                               (size_t)(frame->width * depth));
                     }
                 }
                 frame->multiframe = 1;
-                gdk_pixbuf_animation_iter_advance(it, &time);
+                gdk_pixbuf_animation_iter_advance(it, &time_val);
                 status = fn_load(frame, context);
                 if (status != SIXEL_OK) {
                     goto end;
@@ -1193,7 +1234,7 @@ load_with_gd(
     frame->width = gdImageSX(im);
     frame->height = gdImageSY(im);
     frame->pixelformat = SIXEL_PIXELFORMAT_RGB888;
-    p = frame->pixels = sixel_allocator_malloc(pchunk->allocator, frame->width * frame->height * 3);
+    p = frame->pixels = sixel_allocator_malloc(pchunk->allocator, (size_t)(frame->width * frame->height * 3));
     if (frame->pixels == NULL) {
         sixel_helper_set_additional_message(
             "load_with_gd: sixel_allocator_malloc() failed.");
@@ -1351,7 +1392,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
