@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -97,6 +97,16 @@ get_rgb(unsigned char const *data,
         *g = (pixels >>  8) & 0xff;
         *b = (pixels >>  0) & 0xff;
         break;
+    case SIXEL_PIXELFORMAT_BGRA8888:
+        *r = (pixels >>  8) & 0xff;
+        *g = (pixels >> 16) & 0xff;
+        *b = (pixels >> 24) & 0xff;
+        break;
+    case SIXEL_PIXELFORMAT_ABGR8888:
+        *r = (pixels >>  0) & 0xff;
+        *g = (pixels >>  8) & 0xff;
+        *b = (pixels >> 16) & 0xff;
+        break;
     case SIXEL_PIXELFORMAT_GA88:
         *r = *g = *b = (pixels >> 8) & 0xff;
         break;
@@ -119,6 +129,8 @@ sixel_helper_compute_depth(int pixelformat)
     switch (pixelformat) {
     case SIXEL_PIXELFORMAT_ARGB8888:
     case SIXEL_PIXELFORMAT_RGBA8888:
+    case SIXEL_PIXELFORMAT_ABGR8888:
+    case SIXEL_PIXELFORMAT_BGRA8888:
         depth = 4;
         break;
     case SIXEL_PIXELFORMAT_RGB888:
@@ -273,6 +285,8 @@ sixel_helper_normalize_pixelformat(
         break;
     case SIXEL_PIXELFORMAT_RGBA8888:
     case SIXEL_PIXELFORMAT_ARGB8888:
+    case SIXEL_PIXELFORMAT_BGRA8888:
+    case SIXEL_PIXELFORMAT_ABGR8888:
         expand_rgb(dst, src, width, height, src_pixelformat, 4);
         *dst_pixelformat = SIXEL_PIXELFORMAT_RGB888;
         break;
@@ -295,7 +309,7 @@ sixel_helper_normalize_pixelformat(
         }
         break;
     case SIXEL_PIXELFORMAT_PAL8:
-        memcpy(dst, src, width * height);
+        memcpy(dst, src, (size_t)(width * height));
         *dst_pixelformat = src_pixelformat;
         break;
     default:
@@ -697,7 +711,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
