@@ -22,7 +22,7 @@
  *
  * ******************************************************************************
  *
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -75,9 +75,9 @@ static inline void quant_trace(FILE *f, ...) { (void) f; }
 
 typedef struct box* boxVector;
 struct box {
-    int ind;
-    int colors;
-    int sum;
+    unsigned int ind;
+    unsigned int colors;
+    unsigned int sum;
 };
 
 typedef unsigned long sample;
@@ -88,7 +88,7 @@ struct tupleint {
        would find in a tuple table or tuple hash.
        Note that this is a variable length structure.
     */
-    int value;
+    unsigned int value;
     sample tuple[1];
     /* This is actually a variable size array -- its size is the
        depth of the tuple in question.  Some compilers do not let us
@@ -111,19 +111,22 @@ static int
 compareplane(const void * const arg1,
              const void * const arg2)
 {
+    int lhs, rhs;
+
     typedef const struct tupleint * const * const sortarg;
     sortarg comparandPP  = (sortarg) arg1;
     sortarg comparatorPP = (sortarg) arg2;
+    lhs = (int)(*comparandPP)->tuple[compareplanePlane];
+    rhs = (int)(*comparatorPP)->tuple[compareplanePlane];
 
-    return (*comparandPP)->tuple[compareplanePlane] -
-        (*comparatorPP)->tuple[compareplanePlane];
+    return lhs - rhs;
 }
 
 
 static int
 sumcompare(const void * const b1, const void * const b2)
 {
-    return ((boxVector)b2)->sum - ((boxVector)b1)->sum;
+    return (int)((boxVector)b2)->sum - (int)((boxVector)b1)->sum;
 }
 
 
@@ -235,15 +238,15 @@ end:
 
 static boxVector
 newBoxVector(
-    int const           /* in */ colors,
-    int const           /* in */ sum,
-    int const           /* in */ newcolors,
+    unsigned int const  /* in */ colors,
+    unsigned int const  /* in */ sum,
+    unsigned int const  /* in */ newcolors,
     sixel_allocator_t   /* in */ *allocator)
 {
     boxVector bv;
 
     bv = (boxVector)sixel_allocator_malloc(allocator,
-                                           sizeof(struct box) * newcolors);
+                                           sizeof(struct box) * (size_t)newcolors);
     if (bv == NULL) {
         quant_trace(stderr, "out of memory allocating box vector table\n");
         return NULL;
@@ -279,7 +282,6 @@ findBoxBoundaries(tupletable2  const colorfreqtable,
     }
 
     for (i = 1; i < boxSize; ++i) {
-        unsigned int plane;
         for (plane = 0; plane < depth; ++plane) {
             sample const v = colorfreqtable.table[boxStart + i]->tuple[plane];
             if (v < minval[plane]) minval[plane] = v;
@@ -351,23 +353,22 @@ largestByLuminosity(sample minval[], sample maxval[], unsigned int const depth)
 
 
 static void
-centerBox(int          const boxStart,
-          int          const boxSize,
+centerBox(unsigned int const boxStart,
+          unsigned int const boxSize,
           tupletable2  const colorfreqtable,
           unsigned int const depth,
           tuple        const newTuple)
 {
 
     unsigned int plane;
+    sample minval, maxval;
+    unsigned int i;
 
     for (plane = 0; plane < depth; ++plane) {
-        int minval, maxval;
-        int i;
-
         minval = maxval = colorfreqtable.table[boxStart]->tuple[plane];
 
         for (i = 1; i < boxSize; ++i) {
-            int const v = colorfreqtable.table[boxStart + i]->tuple[plane];
+            sample v = colorfreqtable.table[boxStart + i]->tuple[plane];
             minval = minval < v ? minval: v;
             maxval = maxval > v ? maxval: v;
         }
@@ -378,22 +379,22 @@ centerBox(int          const boxStart,
 
 
 static void
-averageColors(int          const boxStart,
-              int          const boxSize,
+averageColors(unsigned int const boxStart,
+              unsigned int const boxSize,
               tupletable2  const colorfreqtable,
               unsigned int const depth,
               tuple        const newTuple)
 {
     unsigned int plane;
+    sample sum;
+    unsigned int i;
 
     for (plane = 0; plane < depth; ++plane) {
-        sample sum;
-        int i;
-
         sum = 0;
 
-        for (i = 0; i < boxSize; ++i)
-            sum += colorfreqtable.table[boxStart+i]->tuple[plane];
+        for (i = 0; i < boxSize; ++i) {
+            sum += colorfreqtable.table[boxStart + i]->tuple[plane];
+        }
 
         newTuple[plane] = sum / boxSize;
     }
@@ -402,8 +403,8 @@ averageColors(int          const boxStart,
 
 
 static void
-averagePixels(int const boxStart,
-              int const boxSize,
+averagePixels(unsigned int const boxStart,
+              unsigned int const boxSize,
               tupletable2 const colorfreqtable,
               unsigned int const depth,
               tuple const newTuple)
@@ -412,22 +413,23 @@ averagePixels(int const boxStart,
     unsigned int n;
         /* Number of tuples represented by the box */
     unsigned int plane;
-    int i;
+    unsigned int i;
 
     /* Count the tuples in question */
     n = 0;  /* initial value */
-    for (i = 0; i < boxSize; ++i)
-        n += colorfreqtable.table[boxStart + i]->value;
+    for (i = 0; i < boxSize; ++i) {
+        n += (unsigned int)colorfreqtable.table[boxStart + i]->value;
+    }
 
     for (plane = 0; plane < depth; ++plane) {
         sample sum;
-        int i;
 
         sum = 0;
 
-        for (i = 0; i < boxSize; ++i)
-            sum += colorfreqtable.table[boxStart+i]->tuple[plane]
-                * colorfreqtable.table[boxStart+i]->value;
+        for (i = 0; i < boxSize; ++i) {
+            sum += colorfreqtable.table[boxStart + i]->tuple[plane]
+                * (unsigned int)colorfreqtable.table[boxStart + i]->value;
+        }
 
         newTuple[plane] = sum / n;
     }
@@ -509,7 +511,7 @@ splitBox(boxVector const bv,
     unsigned int const boxSize  = bv[bi].colors;
     unsigned int const sm       = bv[bi].sum;
 
-    unsigned int const max_depth = 16;
+    enum { max_depth= 16 };
     sample minval[max_depth];
     sample maxval[max_depth];
 
@@ -657,14 +659,14 @@ end:
 }
 
 
-static int
-computeHash(unsigned char const *data, int const depth)
+static unsigned int
+computeHash(unsigned char const *data, unsigned int const depth)
 {
-    int hash = 0;
-    int n;
+    unsigned int hash = 0;
+    unsigned int n;
 
     for (n = 0; n < depth; n++) {
-        hash |= data[depth - 1 - n] >> 3 << n * 5;
+        hash |= (unsigned int)(data[depth - 1 - n] >> 3) << n * 5;
     }
 
     return hash;
@@ -686,7 +688,7 @@ computeHistogram(unsigned char const    /* in */  *data,
     unit_t *refmap = NULL;
     unit_t *ref;
     unit_t *it;
-    unsigned int index;
+    unsigned int bucket_index;
     unsigned int step;
     unsigned int max_sample;
 
@@ -717,7 +719,7 @@ computeHistogram(unsigned char const    /* in */  *data,
     quant_trace(stderr, "making histogram...\n");
 
     histogram = (unit_t *)sixel_allocator_calloc(allocator,
-                                                 1 << depth * 5,
+                                                 (size_t)(1 << depth * 5),
                                                  sizeof(unit_t));
     if (histogram == NULL) {
         sixel_helper_set_additional_message(
@@ -727,7 +729,7 @@ computeHistogram(unsigned char const    /* in */  *data,
     }
     it = ref = refmap
         = (unsigned short *)sixel_allocator_malloc(allocator,
-                                                   (1 << depth * 5) * sizeof(unit_t));
+                                                   (size_t)(1 << depth * 5) * sizeof(unit_t));
     if (!it) {
         sixel_helper_set_additional_message(
             "unable to allocate memory for lookup table.");
@@ -736,17 +738,17 @@ computeHistogram(unsigned char const    /* in */  *data,
     }
 
     for (i = 0; i < length - depth; i += step) {
-        index = computeHash(data + i, 3);
-        if (histogram[index] == 0) {
-            *ref++ = index;
+        bucket_index = computeHash(data + i, 3);
+        if (histogram[bucket_index] == 0) {
+            *ref++ = bucket_index;
         }
-        if (histogram[index] < (1 << sizeof(unsigned short) * 8) - 1) {
-            histogram[index]++;
+        if (histogram[bucket_index] < (unsigned int)(1 << sizeof(unsigned short) * 8) - 1) {
+            histogram[bucket_index]++;
         }
     }
 
-    colorfreqtableP->size = ref - refmap;
-    status = alloctupletable(&colorfreqtableP->table, depth, ref - refmap, allocator);
+    colorfreqtableP->size = (unsigned int)(ref - refmap);
+    status = alloctupletable(&colorfreqtableP->table, depth, (unsigned int)(ref - refmap), allocator);
     if (SIXEL_FAILED(status)) {
         goto end;
     }
@@ -755,7 +757,7 @@ computeHistogram(unsigned char const    /* in */  *data,
             colorfreqtableP->table[i]->value = histogram[refmap[i]];
             for (n = 0; n < depth; n++) {
                 colorfreqtableP->table[i]->tuple[depth - 1 - n]
-                    = (*it >> n * 5 & 0x1f) << 3;
+                    = (sample)((*it >> n * 5 & 0x1f) << 3);
             }
         }
         it++;
@@ -782,7 +784,7 @@ computeColorMapFromInput(unsigned char const *data,
                          int const methodForRep,
                          int const qualityMode,
                          tupletable2 * const colormapP,
-                         int *origcolors,
+                         unsigned int *origcolors,
                          sixel_allocator_t *allocator)
 {
 /*----------------------------------------------------------------------------
@@ -852,15 +854,20 @@ end:
 }
 
 
+/* diffuse error energy to surround pixels */
 static void
-error_diffuse(unsigned char *data, int pos, int depth,
-              int offset, int mul, int div)
+error_diffuse(unsigned char /* in */    *data,      /* base address of pixel buffer */
+              int           /* in */    pos,        /* address of the destination pixel */
+              int           /* in */    depth,      /* color depth in bytes */
+              int           /* in */    error,      /* error energy */
+              int           /* in */    numerator,  /* numerator of diffusion coefficient */
+              int           /* in */    denominator /* denominator of diffusion coefficient */)
 {
     int c;
 
     data += pos * depth;
 
-    c = *data + offset * mul / div;
+    c = *data + error * numerator / denominator;
     if (c < 0) {
         c = 0;
     }
@@ -873,7 +880,7 @@ error_diffuse(unsigned char *data, int pos, int depth,
 
 static void
 diffuse_none(unsigned char *data, int width, int height,
-             int x, int y, int depth, int offset)
+             int x, int y, int depth, int error)
 {
     /* unused */ (void) data;
     /* unused */ (void) width;
@@ -881,13 +888,13 @@ diffuse_none(unsigned char *data, int width, int height,
     /* unused */ (void) x;
     /* unused */ (void) y;
     /* unused */ (void) depth;
-    /* unused */ (void) offset;
+    /* unused */ (void) error;
 }
 
 
 static void
 diffuse_fs(unsigned char *data, int width, int height,
-           int x, int y, int depth, int offset)
+           int x, int y, int depth, int error)
 {
     int pos;
 
@@ -898,21 +905,21 @@ diffuse_fs(unsigned char *data, int width, int height,
      *  3/16    5/48    1/16
      */
     if (x < width - 1 && y < height - 1) {
-        /* add offset to the right cell */
-        error_diffuse(data, pos + width * 0 + 1, depth, offset, 7, 16);
-        /* add offset to the left-bottom cell */
-        error_diffuse(data, pos + width * 1 - 1, depth, offset, 3, 16);
-        /* add offset to the bottom cell */
-        error_diffuse(data, pos + width * 1 + 0, depth, offset, 5, 16);
-        /* add offset to the right-bottom cell */
-        error_diffuse(data, pos + width * 1 + 1, depth, offset, 1, 16);
+        /* add error to the right cell */
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 16);
+        /* add error to the left-bottom cell */
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 3, 16);
+        /* add error to the bottom cell */
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 5, 16);
+        /* add error to the right-bottom cell */
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 16);
     }
 }
 
 
 static void
 diffuse_atkinson(unsigned char *data, int width, int height,
-                 int x, int y, int depth, int offset)
+                 int x, int y, int depth, int error)
 {
     int pos;
 
@@ -924,25 +931,25 @@ diffuse_atkinson(unsigned char *data, int width, int height,
      *           1/8
      */
     if (y < height - 2) {
-        /* add offset to the right cell */
-        error_diffuse(data, pos + width * 0 + 1, depth, offset, 1, 8);
-        /* add offset to the 2th right cell */
-        error_diffuse(data, pos + width * 0 + 2, depth, offset, 1, 8);
-        /* add offset to the left-bottom cell */
-        error_diffuse(data, pos + width * 1 - 1, depth, offset, 1, 8);
-        /* add offset to the bottom cell */
-        error_diffuse(data, pos + width * 1 + 0, depth, offset, 1, 8);
-        /* add offset to the right-bottom cell */
-        error_diffuse(data, pos + width * 1 + 1, depth, offset, 1, 8);
-        /* add offset to the 2th bottom cell */
-        error_diffuse(data, pos + width * 2 + 0, depth, offset, 1, 8);
+        /* add error to the right cell */
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 8);
+        /* add error to the 2th right cell */
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8);
+        /* add error to the left-bottom cell */
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8);
+        /* add error to the bottom cell */
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 8);
+        /* add error to the right-bottom cell */
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8);
+        /* add error to the 2th bottom cell */
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 8);
     }
 }
 
 
 static void
 diffuse_jajuni(unsigned char *data, int width, int height,
-               int x, int y, int depth, int offset)
+               int x, int y, int depth, int error)
 {
     int pos;
 
@@ -954,25 +961,25 @@ diffuse_jajuni(unsigned char *data, int width, int height,
      *  1/48    3/48    5/48    3/48    1/48
      */
     if (pos < (height - 2) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, offset, 7, 48);
-        error_diffuse(data, pos + width * 0 + 2, depth, offset, 5, 48);
-        error_diffuse(data, pos + width * 1 - 2, depth, offset, 3, 48);
-        error_diffuse(data, pos + width * 1 - 1, depth, offset, 5, 48);
-        error_diffuse(data, pos + width * 1 + 0, depth, offset, 7, 48);
-        error_diffuse(data, pos + width * 1 + 1, depth, offset, 5, 48);
-        error_diffuse(data, pos + width * 1 + 2, depth, offset, 3, 48);
-        error_diffuse(data, pos + width * 2 - 2, depth, offset, 1, 48);
-        error_diffuse(data, pos + width * 2 - 1, depth, offset, 3, 48);
-        error_diffuse(data, pos + width * 2 + 0, depth, offset, 5, 48);
-        error_diffuse(data, pos + width * 2 + 1, depth, offset, 3, 48);
-        error_diffuse(data, pos + width * 2 + 2, depth, offset, 1, 48);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 7, 48);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 5, 48);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 3, 48);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 5, 48);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 7, 48);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 5, 48);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 3, 48);
+        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48);
+        error_diffuse(data, pos + width * 2 - 1, depth, error, 3, 48);
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 5, 48);
+        error_diffuse(data, pos + width * 2 + 1, depth, error, 3, 48);
+        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48);
     }
 }
 
 
 static void
 diffuse_stucki(unsigned char *data, int width, int height,
-               int x, int y, int depth, int offset)
+               int x, int y, int depth, int error)
 {
     int pos;
 
@@ -984,25 +991,25 @@ diffuse_stucki(unsigned char *data, int width, int height,
      *  1/48    2/48    4/48    2/48    1/48
      */
     if (pos < (height - 2) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, offset, 1, 6);
-        error_diffuse(data, pos + width * 0 + 2, depth, offset, 1, 12);
-        error_diffuse(data, pos + width * 1 - 2, depth, offset, 1, 24);
-        error_diffuse(data, pos + width * 1 - 1, depth, offset, 1, 12);
-        error_diffuse(data, pos + width * 1 + 0, depth, offset, 1, 6);
-        error_diffuse(data, pos + width * 1 + 1, depth, offset, 1, 12);
-        error_diffuse(data, pos + width * 1 + 2, depth, offset, 1, 24);
-        error_diffuse(data, pos + width * 2 - 2, depth, offset, 1, 48);
-        error_diffuse(data, pos + width * 2 - 1, depth, offset, 1, 24);
-        error_diffuse(data, pos + width * 2 + 0, depth, offset, 1, 12);
-        error_diffuse(data, pos + width * 2 + 1, depth, offset, 1, 24);
-        error_diffuse(data, pos + width * 2 + 2, depth, offset, 1, 48);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 6);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 12);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 24);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 12);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 6);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 12);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 24);
+        error_diffuse(data, pos + width * 2 - 2, depth, error, 1, 48);
+        error_diffuse(data, pos + width * 2 - 1, depth, error, 1, 24);
+        error_diffuse(data, pos + width * 2 + 0, depth, error, 1, 12);
+        error_diffuse(data, pos + width * 2 + 1, depth, error, 1, 24);
+        error_diffuse(data, pos + width * 2 + 2, depth, error, 1, 48);
     }
 }
 
 
 static void
 diffuse_burkes(unsigned char *data, int width, int height,
-               int x, int y, int depth, int offset)
+               int x, int y, int depth, int error)
 {
     int pos;
 
@@ -1013,17 +1020,18 @@ diffuse_burkes(unsigned char *data, int width, int height,
      *  1/16    2/16    4/16    2/16    1/16
      */
     if (pos < (height - 1) * width - 2) {
-        error_diffuse(data, pos + width * 0 + 1, depth, offset, 1, 4);
-        error_diffuse(data, pos + width * 0 + 2, depth, offset, 1, 8);
-        error_diffuse(data, pos + width * 1 - 2, depth, offset, 1, 16);
-        error_diffuse(data, pos + width * 1 - 1, depth, offset, 1, 8);
-        error_diffuse(data, pos + width * 1 + 0, depth, offset, 1, 4);
-        error_diffuse(data, pos + width * 1 + 1, depth, offset, 1, 8);
-        error_diffuse(data, pos + width * 1 + 2, depth, offset, 1, 16);
+        error_diffuse(data, pos + width * 0 + 1, depth, error, 1, 4);
+        error_diffuse(data, pos + width * 0 + 2, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 - 2, depth, error, 1, 16);
+        error_diffuse(data, pos + width * 1 - 1, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 + 0, depth, error, 1, 4);
+        error_diffuse(data, pos + width * 1 + 1, depth, error, 1, 8);
+        error_diffuse(data, pos + width * 1 + 2, depth, error, 1, 16);
     }
 }
 
 
+/* lookup closest color from palette with "normal" strategy */
 static int
 lookup_normal(unsigned char const * const pixel,
               int const depth,
@@ -1032,14 +1040,14 @@ lookup_normal(unsigned char const * const pixel,
               unsigned short * const cachetable,
               int const complexion)
 {
-    int index;
+    int result;
     int diff;
     int r;
     int i;
     int n;
     int distant;
 
-    index = -1;
+    result = (-1);
     diff = INT_MAX;
 
     /* don't use cachetable in 'normal' strategy */
@@ -1055,14 +1063,15 @@ lookup_normal(unsigned char const * const pixel,
         }
         if (distant < diff) {
             diff = distant;
-            index = i;
+            result = i;
         }
     }
 
-    return index;
+    return result;
 }
 
 
+/* lookup closest color from palette with "fast" strategy */
 static int
 lookup_fast(unsigned char const * const pixel,
             int const depth,
@@ -1071,8 +1080,8 @@ lookup_fast(unsigned char const * const pixel,
             unsigned short * const cachetable,
             int const complexion)
 {
-    int hash;
-    int index;
+    int result;
+    unsigned int hash;
     int diff;
     int cache;
     int i;
@@ -1081,7 +1090,7 @@ lookup_fast(unsigned char const * const pixel,
     /* don't use depth in 'fast' strategy because it's always 3 */
     (void) depth;
 
-    index = -1;
+    result = (-1);
     diff = INT_MAX;
     hash = computeHash(pixel, 3);
 
@@ -1105,12 +1114,12 @@ lookup_fast(unsigned char const * const pixel,
 #endif
         if (distant < diff) {
             diff = distant;
-            index = i;
+            result = i;
         }
     }
-    cachetable[hash] = index + 1;
+    cachetable[hash] = result + 1;
 
-    return index;
+    return result;
 }
 
 
@@ -1160,33 +1169,37 @@ lookup_mono_lightbg(unsigned char const * const pixel,
 }
 
 
+/* choose colors using median-cut method */
 SIXELSTATUS
 sixel_quant_make_palette(
     unsigned char          /* out */ **result,
     unsigned char const    /* in */  *data,
-    int                    /* in */  length,
+    unsigned int           /* in */  length,
     int                    /* in */  pixelformat,
-    int                    /* in */  reqcolors,
-    int                    /* in */  *ncolors,
-    int                    /* in */  *origcolors,
+    unsigned int           /* in */  reqcolors,
+    unsigned int           /* in */  *ncolors,
+    unsigned int           /* in */  *origcolors,
     int                    /* in */  methodForLargest,
     int                    /* in */  methodForRep,
     int                    /* in */  qualityMode,
     sixel_allocator_t      /* in */  *allocator)
 {
     SIXELSTATUS status = SIXEL_FALSE;
-    int i;
-    int n;
+    unsigned int i;
+    unsigned int n;
     int ret;
     tupletable2 colormap;
-    int depth;
+    unsigned int depth;
+    int result_depth;
 
-    depth = sixel_helper_compute_depth(pixelformat);
-    if (depth == (-1)) {
+    result_depth = sixel_helper_compute_depth(pixelformat);
+    if (result_depth <= 0) {
         *result = NULL;
         goto end;
     }
 
+    depth = (unsigned int)result_depth;
+
     ret = computeColorMapFromInput(data, length, depth,
                                    reqcolors, methodForLargest,
                                    methodForRep, qualityMode,
@@ -1213,6 +1226,7 @@ end:
 }
 
 
+/* apply color palette into specified pixel buffers */
 SIXELSTATUS
 sixel_quant_apply_palette(
     unsigned char     /* out */ *result,
@@ -1234,7 +1248,7 @@ sixel_quant_apply_palette(
     SIXELSTATUS status = SIXEL_FALSE;
     int pos, n, x, y, sum1, sum2;
     component_t offset;
-    int index;
+    int color_index;
     unsigned short *indextable;
     unsigned char new_palette[256 * 4];
     unsigned short migration_map[256];
@@ -1305,7 +1319,7 @@ sixel_quant_apply_palette(
     indextable = cachetable;
     if (cachetable == NULL && f_lookup == lookup_fast) {
         indextable = (unsigned short *)sixel_allocator_calloc(allocator,
-                                                              1 << depth * 5,
+                                                              (size_t)(1 << depth * 5),
                                                               sizeof(unsigned short));
         if (!indextable) {
             quant_trace(stderr, "Unable to allocate memory for indextable.\n");
@@ -1322,34 +1336,34 @@ sixel_quant_apply_palette(
         for (y = 0; y < height; ++y) {
             for (x = 0; x < width; ++x) {
                 pos = y * width + x;
-                index = f_lookup(data + (pos * depth), depth,
-                                 palette, reqcolor, indextable, complexion);
-                if (migration_map[index] == 0) {
+                color_index = f_lookup(data + (pos * depth), depth,
+                                       palette, reqcolor, indextable, complexion);
+                if (migration_map[color_index] == 0) {
                     result[pos] = *ncolors;
                     for (n = 0; n < depth; ++n) {
-                        new_palette[*ncolors * depth + n] = palette[index * depth + n];
+                        new_palette[*ncolors * depth + n] = palette[color_index * depth + n];
                     }
                     ++*ncolors;
-                    migration_map[index] = *ncolors;
+                    migration_map[color_index] = *ncolors;
                 } else {
-                    result[pos] = migration_map[index] - 1;
+                    result[pos] = migration_map[color_index] - 1;
                 }
                 for (n = 0; n < depth; ++n) {
-                    offset = data[pos * depth + n] - palette[index * depth + n];
+                    offset = data[pos * depth + n] - palette[color_index * depth + n];
                     f_diffuse(data + n, width, height, x, y, depth, offset);
                 }
             }
         }
-        memcpy(palette, new_palette, *ncolors * depth);
+        memcpy(palette, new_palette, (size_t)(*ncolors * depth));
     } else {
         for (y = 0; y < height; ++y) {
             for (x = 0; x < width; ++x) {
                 pos = y * width + x;
-                index = f_lookup(data + (pos * depth), depth,
+                color_index = f_lookup(data + (pos * depth), depth,
                                  palette, reqcolor, indextable, complexion);
-                result[pos] = index;
+                result[pos] = color_index;
                 for (n = 0; n < depth; ++n) {
-                    offset = data[pos * depth + n] - palette[index * depth + n];
+                    offset = data[pos * depth + n] - palette[color_index * depth + n];
                     f_diffuse(data + n, width, height, x, y, depth, offset);
                 }
             }
@@ -1422,8 +1436,11 @@ error:
 }
 #endif  /* HAVE_TESTS */
 
-
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
