@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014,2015 Hayaki Saito
+ * Copyright (c) 2014-2016 Hayaki Saito
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -28,28 +28,30 @@ extern "C" {
 
 #include <sixel.h>
 
+/* choose colors using median-cut method */
 SIXELSTATUS
 sixel_quant_make_palette(
     unsigned char           /* out */ **result,
     unsigned const char     /* in */  *data,             /* data for sampling */
-    int                     /* in */  length,            /* data size */
-    int                     /* in */  depth,
-    int                     /* in */  reqcolors,
-    int                     /* in */  *ncolors,
-    int                     /* in */  *origcolors,
+    unsigned int            /* in */  length,            /* data size */
+    int                     /* in */  pixelformat,
+    unsigned int            /* in */  reqcolors,
+    unsigned int            /* in */  *ncolors,
+    unsigned int            /* in */  *origcolors,
     int                     /* in */  methodForLargest,
     int                     /* in */  methodForRep,
     int                     /* in */  qualityMode,
     sixel_allocator_t       /* in */  *allocator);
 
 
+/* apply color palette into specified pixel buffers */
 SIXELSTATUS
 sixel_quant_apply_palette(
     unsigned char       /* out */ *result,
     unsigned char       /* in */  *data,
     int                 /* in */  width,
     int                 /* in */  height,
-    int                 /* in */  depth,
+    int                 /* in */  pixelformat,
     unsigned char       /* in */  *palette,
     int                 /* in */  reqcolor,
     int const           /* in */  methodForDiffuse,
@@ -61,6 +63,7 @@ sixel_quant_apply_palette(
     sixel_allocator_t   /* in */  *allocator);
 
 
+/* deallocate specified palette */
 void
 sixel_quant_free_palette(
     unsigned char       /* in */ *data,
@@ -77,6 +80,11 @@ sixel_quant_tests_main(void);
 
 #endif /* LIBSIXEL_QUANT_H */
 
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
