@@ -49,61 +49,63 @@ enum {
 
 /* implementation */
 
+/* GNU Screen penetration */
 static void
-penetrate(sixel_output_t *context,
-          int nwrite,
-          char const *dcs_start,
-          char const *dcs_end,
-          int const dcs_start_size,
-          int const dcs_end_size)
+sixel_penetrate(
+    sixel_output_t  /* in */    *output,        /* output context */
+    int             /* in */    nwrite,         /* output size */
+    char const      /* in */    *dcs_start,     /* DCS introducer */
+    char const      /* in */    *dcs_end,       /* DCS terminator */
+    int const       /* in */    dcs_start_size, /* size of DCS introducer */
+    int const       /* in */    dcs_end_size)   /* size of DCS terminator */
 {
     int pos;
     int const splitsize = SCREEN_PACKET_SIZE
                         - dcs_start_size - dcs_end_size;
 
     for (pos = 0; pos < nwrite; pos += splitsize) {
-        context->fn_write((char *)dcs_start, dcs_end_size, context->priv);
-        context->fn_write(((char *)context->buffer) + pos,
+        output->fn_write((char *)dcs_start, dcs_end_size, output->priv);
+        output->fn_write(((char *)output->buffer) + pos,
                           nwrite - pos < splitsize ? nwrite - pos: splitsize,
-                          context->priv);
-        context->fn_write((char *)dcs_end, dcs_end_size, context->priv);
+                          output->priv);
+        output->fn_write((char *)dcs_end, dcs_end_size, output->priv);
     }
 }
 
 
 static void
-sixel_advance(sixel_output_t *context, int nwrite)
+sixel_advance(sixel_output_t *output, int nwrite)
 {
-    if ((context->pos += nwrite) >= SIXEL_OUTPUT_PACKET_SIZE) {
-        if (context->penetrate_multiplexer) {
-            penetrate(context,
-                      SIXEL_OUTPUT_PACKET_SIZE,
-                      DCS_START_7BIT,
-                      DCS_END_7BIT,
-                      DCS_START_7BIT_SIZE,
-                      DCS_END_7BIT_SIZE);
+    if ((output->pos += nwrite) >= SIXEL_OUTPUT_PACKET_SIZE) {
+        if (output->penetrate_multiplexer) {
+            sixel_penetrate(output,
+                            SIXEL_OUTPUT_PACKET_SIZE,
+                            DCS_START_7BIT,
+                            DCS_END_7BIT,
+                            DCS_START_7BIT_SIZE,
+                            DCS_END_7BIT_SIZE);
         } else {
-            context->fn_write((char *)context->buffer,
-                              SIXEL_OUTPUT_PACKET_SIZE, context->priv);
+            output->fn_write((char *)output->buffer,
+                             SIXEL_OUTPUT_PACKET_SIZE, output->priv);
         }
-        memcpy(context->buffer,
-               context->buffer + SIXEL_OUTPUT_PACKET_SIZE,
-               (context->pos -= SIXEL_OUTPUT_PACKET_SIZE));
+        memcpy(output->buffer,
+               output->buffer + SIXEL_OUTPUT_PACKET_SIZE,
+               (size_t)(output->pos -= SIXEL_OUTPUT_PACKET_SIZE));
     }
 }
 
 
 static void
-sixel_putc(char *buffer, unsigned char value)
+sixel_putc(unsigned char *buffer, unsigned char value)
 {
     *buffer = value;
 }
 
 
 static void
-sixel_puts(char *buffer, char const *value, int size)
+sixel_puts(unsigned char *buffer, char const *value, int size)
 {
-    memcpy(buffer, (void *)value, size);
+    memcpy(buffer, (void *)value, (size_t)size);
 }
 
 
@@ -124,7 +126,7 @@ sixel_putnum_impl(char *buffer, long value, int pos)
 
 
 static int
-sixel_putnum(char *buffer, unsigned int value)
+sixel_putnum(char *buffer, int value)
 {
     int pos;
 
@@ -139,45 +141,46 @@ sixel_putnum(char *buffer, unsigned int value)
 
 
 static SIXELSTATUS
-sixel_put_flash(sixel_output_t *const context)
+sixel_put_flash(sixel_output_t *const output)
 {
     int n;
     int nwrite;
 
-#if defined(USE_VT240)        /* VT240 Max 255 ? */
-    while (context->save_count > 255) {
-        sixel_putstr(context->buffer + context->pos, "!255", 4);
-        sixel_advance(context, 4);
-        sixel_putc((char *)context->buffer + context->pos, context->save_pixel);
-        sixel_advance(context, 1);
-        context->save_count -= 255;
-    }
-#endif  /* defined(USE_VT240) */
-
-    if (context->save_count > 3) {
-        /* DECGRI Graphics Repeat Introducer ! Pn Ch */
-        sixel_putc((char *)context->buffer + context->pos, '!');
-        sixel_advance(context, 1);
-        nwrite = sixel_putnum((char *)context->buffer + context->pos, context->save_count);
-        sixel_advance(context, nwrite);
-        sixel_putc((char *)context->buffer + context->pos, context->save_pixel);
-        sixel_advance(context, 1);
-    } else {
-        for (n = 0; n < context->save_count; n++) {
-            context->buffer[context->pos] = (char)context->save_pixel;
-            sixel_advance(context, 1);
+    if (output->has_gri_arg_limit) {  /* VT240 Max 255 ? */
+        while (output->save_count > 255) {
+            /* argument of DECGRI('!') is limitted to 255 in real VT */
+            sixel_puts(output->buffer + output->pos, "!255", 4);
+            sixel_advance(output, 4);
+            sixel_putc(output->buffer + output->pos, output->save_pixel);
+            sixel_advance(output, 1);
+            output->save_count -= 255;
         }
     }
 
-    context->save_pixel = 0;
-    context->save_count = 0;
+    if (output->save_count > 3) {
+        /* DECGRI Graphics Repeat Introducer ! Pn Ch */
+        sixel_putc(output->buffer + output->pos, '!');
+        sixel_advance(output, 1);
+        nwrite = sixel_putnum((char *)output->buffer + output->pos, output->save_count);
+        sixel_advance(output, nwrite);
+        sixel_putc(output->buffer + output->pos, output->save_pixel);
+        sixel_advance(output, 1);
+    } else {
+        for (n = 0; n < output->save_count; n++) {
+            output->buffer[output->pos] = output->save_pixel;
+            sixel_advance(output, 1);
+        }
+    }
+
+    output->save_pixel = 0;
+    output->save_count = 0;
 
     return 0;
 }
 
 
 static SIXELSTATUS
-sixel_put_pixel(sixel_output_t *const context, int pix)
+sixel_put_pixel(sixel_output_t *const output, int pix)
 {
     SIXELSTATUS status = SIXEL_FALSE;
 
@@ -187,15 +190,15 @@ sixel_put_pixel(sixel_output_t *const context, int pix)
 
     pix += '?';
 
-    if (pix == context->save_pixel) {
-        context->save_count++;
+    if (pix == output->save_pixel) {
+        output->save_count++;
     } else {
-        status = sixel_put_flash(context);
+        status = sixel_put_flash(output);
         if (SIXEL_FAILED(status)) {
             goto end;
         }
-        context->save_pixel = pix;
-        context->save_count = 1;
+        output->save_pixel = pix;
+        output->save_count = 1;
     }
 
     status = SIXEL_OK;
@@ -204,14 +207,33 @@ end:
     return status;
 }
 
+static SIXELSTATUS
+sixel_node_new(sixel_node_t **np, sixel_allocator_t *allocator)
+{
+    SIXELSTATUS status = SIXEL_FALSE;
+
+    *np = (sixel_node_t *)sixel_allocator_malloc(allocator,
+                                                 sizeof(sixel_node_t));
+    if (np == NULL) {
+        sixel_helper_set_additional_message(
+            "sixel_node_new: sixel_allocator_malloc() failed.");
+        status = SIXEL_BAD_ALLOCATION;
+        goto end;
+    }
+
+    status = SIXEL_OK;
+
+end:
+    return status;
+}
 
 static void
-sixel_node_del(sixel_output_t *const context, sixel_node_t *np)
+sixel_node_del(sixel_output_t *output, sixel_node_t *np)
 {
     sixel_node_t *tp;
 
-    if ((tp = context->node_top) == np) {
-        context->node_top = np->next;
+    if ((tp = output->node_top) == np) {
+        output->node_top = np->next;
     } else {
         while (tp->next != NULL) {
             if (tp->next == np) {
@@ -222,35 +244,36 @@ sixel_node_del(sixel_output_t *const context, sixel_node_t *np)
         }
     }
 
-    np->next = context->node_free;
-    context->node_free = np;
+    np->next = output->node_free;
+    output->node_free = np;
 }
 
 
 static SIXELSTATUS
-sixel_put_node(sixel_output_t *const context,
-               int *x,
-               sixel_node_t *np,
-               int ncolors,
-               int keycolor)
+sixel_put_node(
+    sixel_output_t /* in */     *output,  /* output context */
+    int            /* in/out */ *x,       /* header position */
+    sixel_node_t   /* in */     *np,      /* node object */
+    int            /* in */     ncolors,  /* number of palette colors */
+    int            /* in */     keycolor) /* transparent color number */
 {
     SIXELSTATUS status = SIXEL_FALSE;
     int nwrite;
 
-    if (ncolors != 2 || keycolor == -1) {
+    if (ncolors != 2 || keycolor == (-1)) {
         /* designate palette index */
-        if (context->active_palette != np->pal) {
-            sixel_putc((char *)context->buffer + context->pos, '#');
-            sixel_advance(context, 1);
-            nwrite = sixel_putnum((char *)context->buffer + context->pos, np->pal);
-            sixel_advance(context, nwrite);
-            context->active_palette = np->pal;
+        if (output->active_palette != np->pal) {
+            sixel_putc(output->buffer + output->pos, '#');
+            sixel_advance(output, 1);
+            nwrite = sixel_putnum((char *)output->buffer + output->pos, np->pal);
+            sixel_advance(output, nwrite);
+            output->active_palette = np->pal;
         }
     }
 
     for (; *x < np->sx; ++*x) {
         if (*x != keycolor) {
-            status = sixel_put_pixel(context, 0);
+            status = sixel_put_pixel(output, 0);
             if (SIXEL_FAILED(status)) {
                 goto end;
             }
@@ -259,14 +282,14 @@ sixel_put_node(sixel_output_t *const context,
 
     for (; *x < np->mx; ++*x) {
         if (*x != keycolor) {
-            status = sixel_put_pixel(context, np->map[*x]);
+            status = sixel_put_pixel(output, np->map[*x]);
             if (SIXEL_FAILED(status)) {
                 goto end;
             }
         }
     }
 
-    status = sixel_put_flash(context);
+    status = sixel_put_flash(output);
     if (SIXEL_FAILED(status)) {
         goto end;
     }
@@ -277,7 +300,7 @@ end:
 
 
 static SIXELSTATUS
-sixel_encode_header(int width, int height, sixel_output_t *context)
+sixel_encode_header(int width, int height, sixel_output_t *output)
 {
     SIXELSTATUS status = SIXEL_FALSE;
     int nwrite;
@@ -285,19 +308,19 @@ sixel_encode_header(int width, int height, sixel_output_t *context)
     int pcount = 3;
     int use_raster_attributes = 1;
 
-    context->pos = 0;
+    output->pos = 0;
 
-    if (!context->skip_dcs_envelope) {
-        if (context->has_8bit_control) {
-            sixel_puts((char *)context->buffer + context->pos,
+    if (!output->skip_dcs_envelope) {
+        if (output->has_8bit_control) {
+            sixel_puts(output->buffer + output->pos,
                        DCS_START_8BIT,
                        DCS_START_8BIT_SIZE);
-            sixel_advance(context, DCS_START_8BIT_SIZE);
+            sixel_advance(output, DCS_START_8BIT_SIZE);
         } else {
-            sixel_puts((char *)context->buffer + context->pos,
+            sixel_puts(output->buffer + output->pos,
                        DCS_START_7BIT,
                        DCS_START_7BIT_SIZE);
-            sixel_advance(context, DCS_START_7BIT_SIZE);
+            sixel_advance(output, DCS_START_7BIT_SIZE);
         }
     }
 
@@ -312,34 +335,34 @@ sixel_encode_header(int width, int height, sixel_output_t *context)
     }
 
     if (pcount > 0) {
-        nwrite = sixel_putnum((char *)context->buffer + context->pos, p[0]);
-        sixel_advance(context, nwrite);
+        nwrite = sixel_putnum((char *)output->buffer + output->pos, p[0]);
+        sixel_advance(output, nwrite);
         if (pcount > 1) {
-            sixel_putc((char *)context->buffer + context->pos, ';');
-            sixel_advance(context, 1);
-            nwrite = sixel_putnum((char *)context->buffer + context->pos, p[1]);
-            sixel_advance(context, nwrite);
+            sixel_putc(output->buffer + output->pos, ';');
+            sixel_advance(output, 1);
+            nwrite = sixel_putnum((char *)output->buffer + output->pos, p[1]);
+            sixel_advance(output, nwrite);
             if (pcount > 2) {
-                sixel_putc((char *)context->buffer + context->pos, ';');
-                sixel_advance(context, 1);
-                nwrite = sixel_putnum((char *)context->buffer + context->pos, p[2]);
-                sixel_advance(context, nwrite);
+                sixel_putc(output->buffer + output->pos, ';');
+                sixel_advance(output, 1);
+                nwrite = sixel_putnum((char *)output->buffer + output->pos, p[2]);
+                sixel_advance(output, nwrite);
             }
         }
     }
 
-    sixel_putc((char *)context->buffer + context->pos, 'q');
-    sixel_advance(context, 1);
+    sixel_putc(output->buffer + output->pos, 'q');
+    sixel_advance(output, 1);
 
     if (use_raster_attributes) {
-        sixel_puts((char *)context->buffer + context->pos, "\"1;1;", 5);
-        sixel_advance(context, 5);
-        nwrite = sixel_putnum((char *)context->buffer + context->pos, width);
-        sixel_advance(context, nwrite);
-        sixel_putc((char *)context->buffer + context->pos, ';');
-        sixel_advance(context, 1);
-        nwrite = sixel_putnum((char *)context->buffer + context->pos, height);
-        sixel_advance(context, nwrite);
+        sixel_puts(output->buffer + output->pos, "\"1;1;", 5);
+        sixel_advance(output, 5);
+        nwrite = sixel_putnum((char *)output->buffer + output->pos, width);
+        sixel_advance(output, nwrite);
+        sixel_putc(output->buffer + output->pos, ';');
+        sixel_advance(output, 1);
+        nwrite = sixel_putnum((char *)output->buffer + output->pos, height);
+        sixel_advance(output, nwrite);
     }
 
     status = SIXEL_OK;
@@ -361,21 +384,21 @@ output_rgb_palette_definition(
 
     if (n != keycolor) {
         /* DECGCI Graphics Color Introducer  # Pc ; Pu; Px; Py; Pz */
-        sixel_putc((char *)output->buffer + output->pos, '#');
+        sixel_putc(output->buffer + output->pos, '#');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos, n);
         sixel_advance(output, nwrite);
-        sixel_puts((char *)output->buffer + output->pos, ";2;", 3);
+        sixel_puts(output->buffer + output->pos, ";2;", 3);
         sixel_advance(output, 3);
         nwrite = sixel_putnum((char *)output->buffer + output->pos,
                               (palette[n * 3 + 0] * 100 + 127) / 255);
         sixel_advance(output, nwrite);
-        sixel_putc((char *)output->buffer + output->pos, ';');
+        sixel_putc(output->buffer + output->pos, ';');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos,
                               (palette[n * 3 + 1] * 100 + 127) / 255);
         sixel_advance(output, nwrite);
-        sixel_putc((char *)output->buffer + output->pos, ';');
+        sixel_putc(output->buffer + output->pos, ';');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos,
                               (palette[n * 3 + 2] * 100 + 127) / 255);
@@ -433,19 +456,19 @@ output_hls_palette_definition(
             }
         }
         /* DECGCI Graphics Color Introducer  # Pc ; Pu; Px; Py; Pz */
-        sixel_putc((char *)output->buffer + output->pos, '#');
+        sixel_putc(output->buffer + output->pos, '#');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos, n);
         sixel_advance(output, nwrite);
-        sixel_puts((char *)output->buffer + output->pos, ";1;", 3);
+        sixel_puts(output->buffer + output->pos, ";1;", 3);
         sixel_advance(output, 3);
         nwrite = sixel_putnum((char *)output->buffer + output->pos, h);
         sixel_advance(output, nwrite);
-        sixel_putc((char *)output->buffer + output->pos, ';');
+        sixel_putc(output->buffer + output->pos, ';');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos, l);
         sixel_advance(output, nwrite);
-        sixel_putc((char *)output->buffer + output->pos, ';');
+        sixel_putc(output->buffer + output->pos, ';');
         sixel_advance(output, 1);
         nwrite = sixel_putnum((char *)output->buffer + output->pos, s);
         sixel_advance(output, nwrite);
@@ -491,7 +514,7 @@ sixel_encode_body(
     output->active_palette = (-1);
 
     map = (unsigned char *)sixel_allocator_calloc(allocator,
-                                                  len,
+                                                  (size_t)len,
                                                   sizeof(unsigned char));
     if (map == NULL) {
         sixel_helper_set_additional_message(
@@ -521,11 +544,10 @@ sixel_encode_body(
     for (y = i = 0; y < height; y++) {
         if (output->encode_policy != SIXEL_ENCODEPOLICY_SIZE) {
             fillable = 0;
-        }
-        else if (palstate) {
+        } else if (palstate) {
             /* high color sixel */
             pix = pixels[(y - i) * width];
-            if (pix < 0 || pix >= ncolors || pix == keycolor) {
+            if (pix >= ncolors) {
                 fillable = 0;
             } else {
                 fillable = 1;
@@ -535,7 +557,7 @@ sixel_encode_body(
             fillable = 1;
         }
         for (x = 0; x < width; x++) {
-            pix = pixels[y * width + x];
+            pix = pixels[y * width + x];  /* color index */
             if (pix >= 0 && pix < ncolors && pix != keycolor) {
                 map[pix * width + x] |= (1 << i);
             }
@@ -574,12 +596,8 @@ sixel_encode_body(
                 if ((np = output->node_free) != NULL) {
                     output->node_free = np->next;
                 } else {
-                    np = (sixel_node_t *)sixel_allocator_malloc(allocator,
-                                                                sizeof(sixel_node_t));
-                    if (np == NULL) {
-                        sixel_helper_set_additional_message(
-                            "sixel_encode_body: sixel_allocator_malloc() failed.");
-                        status = SIXEL_BAD_ALLOCATION;
+                    status = sixel_node_new(&np, allocator);
+                    if (SIXEL_FAILED(status)) {
                         goto end;
                     }
                 }
@@ -626,7 +644,7 @@ sixel_encode_body(
             }
 
             if (fillable) {
-                memset(np->map + np->sx, (1 << i) - 1, np->mx - np->sx);
+                memset(np->map + np->sx, (1 << i) - 1, (size_t)(np->mx - np->sx));
             }
             status = sixel_put_node(output, &x, np, ncolors, keycolor);
             if (SIXEL_FAILED(status)) {
@@ -643,7 +661,7 @@ sixel_encode_body(
                 }
 
                 if (fillable) {
-                    memset(np->map + np->sx, (1 << i) - 1, np->mx - np->sx);
+                    memset(np->map + np->sx, (1 << i) - 1, (size_t)(np->mx - np->sx));
                 }
                 status = sixel_put_node(output, &x, np, ncolors, keycolor);
                 if (SIXEL_FAILED(status)) {
@@ -658,7 +676,7 @@ sixel_encode_body(
         }
 
         i = 0;
-        memset(map, 0, len);
+        memset(map, 0, (size_t)len);
     }
 
     if (palstate) {
@@ -689,11 +707,11 @@ sixel_encode_footer(sixel_output_t *output)
 
     if (!output->skip_dcs_envelope && !output->penetrate_multiplexer) {
         if (output->has_8bit_control) {
-            sixel_puts((char *)output->buffer + output->pos,
+            sixel_puts(output->buffer + output->pos,
                        DCS_END_8BIT, DCS_END_8BIT_SIZE);
             sixel_advance(output, DCS_END_8BIT_SIZE);
         } else {
-            sixel_puts((char *)output->buffer + output->pos,
+            sixel_puts(output->buffer + output->pos,
                        DCS_END_7BIT, DCS_END_7BIT_SIZE);
             sixel_advance(output, DCS_END_7BIT_SIZE);
         }
@@ -702,16 +720,15 @@ sixel_encode_footer(sixel_output_t *output)
     /* flush buffer */
     if (output->pos > 0) {
         if (output->penetrate_multiplexer) {
-            penetrate(output, output->pos,
-                      DCS_START_7BIT,
-                      DCS_END_7BIT,
-                      DCS_START_7BIT_SIZE,
-                      DCS_END_7BIT_SIZE);
+            sixel_penetrate(output, output->pos,
+                            DCS_START_7BIT,
+                            DCS_END_7BIT,
+                            DCS_START_7BIT_SIZE,
+                            DCS_END_7BIT_SIZE);
             output->fn_write((char *)DCS_7BIT("\033") DCS_7BIT("\\"),
-                              (DCS_START_7BIT_SIZE + 1 + DCS_END_7BIT_SIZE) * 2,
-                              output->priv);
-        }
-        else {
+                             (DCS_START_7BIT_SIZE + 1 + DCS_END_7BIT_SIZE) * 2,
+                             output->priv);
+        } else {
             output->fn_write((char *)output->buffer, output->pos, output->priv);
         }
     }
@@ -724,11 +741,11 @@ sixel_encode_footer(sixel_output_t *output)
 
 static SIXELSTATUS
 sixel_encode_dither(
-    unsigned char   /* in */ *pixels,
-    int             /* in */ width,
-    int             /* in */ height,
-    sixel_dither_t  /* in */ *dither,
-    sixel_output_t  /* in */ *output)
+    unsigned char   /* in */ *pixels,   /* pixel bytes to be encoded */
+    int             /* in */ width,     /* width of source image */
+    int             /* in */ height,    /* height of source image */
+    sixel_dither_t  /* in */ *dither,   /* dither context */
+    sixel_output_t  /* in */ *output)   /* output context */
 {
     SIXELSTATUS status = SIXEL_FALSE;
     unsigned char *paletted_pixels = NULL;
@@ -738,8 +755,11 @@ sixel_encode_dither(
     case SIXEL_PIXELFORMAT_PAL1:
     case SIXEL_PIXELFORMAT_PAL2:
     case SIXEL_PIXELFORMAT_PAL4:
+    case SIXEL_PIXELFORMAT_G1:
+    case SIXEL_PIXELFORMAT_G2:
+    case SIXEL_PIXELFORMAT_G4:
         paletted_pixels = (unsigned char *)sixel_allocator_malloc(dither->allocator,
-                                                                  width * height * 3);
+                                                                  (size_t)(width * height * 3));
         if (paletted_pixels == NULL) {
             sixel_helper_set_additional_message(
                 "sixel_encode_dither: sixel_allocator_malloc() failed.");
@@ -1189,8 +1209,10 @@ sixel_apply_15bpp_dither(
 
 
 static SIXELSTATUS
-sixel_encode_highcolor(unsigned char *pixels, int width, int height,
-                       sixel_dither_t *dither, sixel_output_t *output)
+sixel_encode_highcolor(
+        unsigned char *pixels, int width, int height,
+        sixel_dither_t *dither, sixel_output_t *output
+        )
 {
     SIXELSTATUS status = SIXEL_FALSE;
     unsigned char *paletted_pixels = NULL;
@@ -1207,11 +1229,21 @@ sixel_encode_highcolor(unsigned char *pixels, int width, int height,
                    + maxcolors       /* for rgbhit */
                    + maxcolors       /* for rgb2pal */
                    + width * 6;      /* for marks */
+    int x, y;
+    unsigned char *dst;
+    unsigned char *mptr;
+    int dirty;
+    int mod_y;
+    int nextpal;
+    int threshold;
+    int pix;
+    int orig_height;
+    unsigned char *pal;
 
     if (dither->pixelformat != SIXEL_PIXELFORMAT_RGB888) {
         /* normalize pixelfromat */
         normalized_pixels = (unsigned char *)sixel_allocator_malloc(dither->allocator,
-                                                                    width * height * 3);
+                                                                    (size_t)(width * height * 3));
         if (normalized_pixels == NULL) {
             goto error;
         }
@@ -1226,145 +1258,127 @@ sixel_encode_highcolor(unsigned char *pixels, int width, int height,
         pixels = normalized_pixels;
     }
     paletted_pixels = (unsigned char *)sixel_allocator_malloc(dither->allocator,
-                                                              whole_size);
+                                                              (size_t)whole_size);
     if (paletted_pixels == NULL) {
         goto error;
     }
     rgbhit = paletted_pixels + width * height;
-    memset(rgbhit, 0, maxcolors * 2 + width * 6);
+    memset(rgbhit, 0, (size_t)(maxcolors * 2 + width * 6));
     rgb2pal = rgbhit + maxcolors;
     marks = rgb2pal + maxcolors;
     output_count = 0;
 
+next:
+    dst = paletted_pixels;
+    nextpal = 0;
+    threshold = 1;
+    dirty = 0;
+    mptr = marks;
+    memset(palstate, 0, sizeof(palstate));
+    y = mod_y = 0;
+
     while (1) {
-        int x, y;
-        unsigned char *dst;
-        unsigned char *mptr;
-        int dirty;
-        int mod_y;
-        int nextpal;
-        int threshold;
-        int pix;
-        int orig_height;
-        unsigned char *pal;
-
-        dst = paletted_pixels;
-        nextpal = 0;
-        threshold = 1;
-        dirty = 0;
-        mptr = marks;
-        memset(palstate, 0, sizeof(palstate));
-        y = mod_y = 0;
-
-        while (1) {
-            for (x = 0; x < width; x++, mptr++, dst++, pixels += 3) {
-                if (*mptr) {
-                    *dst = 255;
-                }
-                else {
-                    pix = ((pixels[0] & 0xf8) << 7) |
-                          ((pixels[1] & 0xf8) << 2) |
-                          ((pixels[2] >> 3) & 0x1f);
-                    sixel_apply_15bpp_dither(pixels,
-                                             x, y, width, height,
-                                             dither->method_for_diffuse);
-                    if (!rgbhit[pix]) {
-                        while (1) {
-                            if (nextpal >= 255) {
-                                if (threshold >= 255) {
-                                    break;
-                                }
-                                else {
-                                    threshold = (threshold == 1) ? 9: 255;
-                                    nextpal = 0;
-                                }
-                            }
-                            else if (palstate[nextpal] ||
-                                     palhitcount[nextpal] > threshold) {
-                                nextpal++;
-                            }
-                            else {
-                                break;
-                            }
-                        }
-
+        for (x = 0; x < width; x++, mptr++, dst++, pixels += 3) {
+            if (*mptr) {
+                *dst = 255;
+            } else {
+                pix = ((pixels[0] & 0xf8) << 7) |
+                      ((pixels[1] & 0xf8) << 2) |
+                      ((pixels[2] >> 3) & 0x1f);
+                sixel_apply_15bpp_dither(pixels,
+                                         x, y, width, height,
+                                         dither->method_for_diffuse);
+                if (!rgbhit[pix]) {
+                    while (1) {
                         if (nextpal >= 255) {
-                            dirty = 1;
-                            *dst = 255;
-                        }
-                        else {
-                            pal = dither->palette + (nextpal * 3);
-
-                            rgbhit[pix] = 1;
-                            if (output_count > 0) {
-                                rgbhit[((pal[0] & 0xf8) << 7) |
-                                       ((pal[1] & 0xf8) << 2) |
-                                       ((pal[2] >> 3) & 0x1f)] = 0;
+                            if (threshold >= 255) {
+                                break;
+                            } else {
+                                threshold = (threshold == 1) ? 9: 255;
+                                nextpal = 0;
                             }
-                            *dst = rgb2pal[pix] = nextpal++;
-                            *mptr = 1;
-                            palstate[*dst] = PALETTE_CHANGE;
-                            palhitcount[*dst] = 1;
-                            *(pal++) = pixels[0];
-                            *(pal++) = pixels[1];
-                            *(pal++) = pixels[2];
+                        } else if (palstate[nextpal] ||
+                                 palhitcount[nextpal] > threshold) {
+                            nextpal++;
+                        } else {
+                            break;
                         }
                     }
-                    else {
-                        *dst = rgb2pal[pix];
+
+                    if (nextpal >= 255) {
+                        dirty = 1;
+                        *dst = 255;
+                    } else {
+                        pal = dither->palette + (nextpal * 3);
+
+                        rgbhit[pix] = 1;
+                        if (output_count > 0) {
+                            rgbhit[((pal[0] & 0xf8) << 7) |
+                                   ((pal[1] & 0xf8) << 2) |
+                                   ((pal[2] >> 3) & 0x1f)] = 0;
+                        }
+                        *dst = rgb2pal[pix] = nextpal++;
                         *mptr = 1;
-                        if (!palstate[*dst]) {
-                            palstate[*dst] = PALETTE_HIT;
-                        }
-                        if (palhitcount[*dst] < 255) {
-                            palhitcount[*dst]++;
-                        }
+                        palstate[*dst] = PALETTE_CHANGE;
+                        palhitcount[*dst] = 1;
+                        *(pal++) = pixels[0];
+                        *(pal++) = pixels[1];
+                        *(pal++) = pixels[2];
+                    }
+                } else {
+                    *dst = rgb2pal[pix];
+                    *mptr = 1;
+                    if (!palstate[*dst]) {
+                        palstate[*dst] = PALETTE_HIT;
+                    }
+                    if (palhitcount[*dst] < 255) {
+                        palhitcount[*dst]++;
                     }
                 }
             }
+        }
 
-            if (++y >= height) {
-                if (dirty) {
-                    mod_y = 5;
-                }
-                else {
-                    goto end;
-                }
+        if (++y >= height) {
+            if (dirty) {
+                mod_y = 5;
+            } else {
+                goto end;
             }
-            if (dirty && mod_y == 5) {
-                orig_height = height;
+        }
+        if (dirty && mod_y == 5) {
+            orig_height = height;
 
-                if (output_count++ == 0) {
-                    status = sixel_encode_header(width, height, output);
-                    if (SIXEL_FAILED(status)) {
-                        goto error;
-                    }
-                }
-                height = y;
-                status = sixel_encode_body(paletted_pixels,
-                                           width,
-                                           height,
-                                           dither->palette,
-                                           255,
-                                           255,
-                                           dither->bodyonly,
-                                           output,
-                                           palstate,
-                                           dither->allocator);
+            if (output_count++ == 0) {
+                status = sixel_encode_header(width, height, output);
                 if (SIXEL_FAILED(status)) {
                     goto error;
                 }
-                pixels -= (6 * width * 3);
-                height = orig_height - height + 6;
-                break;
             }
+            height = y;
+            status = sixel_encode_body(paletted_pixels,
+                                       width,
+                                       height,
+                                       dither->palette,
+                                       255,
+                                       255,
+                                       dither->bodyonly,
+                                       output,
+                                       palstate,
+                                       dither->allocator);
+            if (SIXEL_FAILED(status)) {
+                goto error;
+            }
+            pixels -= (6 * width * 3);
+            height = orig_height - height + 6;
+            goto next;
+        }
 
-            if (++mod_y == 6) {
-                mptr = (unsigned char *)memset(marks, 0, width * 6);
-                mod_y = 0;
-            }
+        if (++mod_y == 6) {
+            mptr = (unsigned char *)memset(marks, 0, (size_t)(width * 6));
+            mod_y = 0;
         }
     }
+    goto next;
 
 end:
     if (output_count == 0) {
@@ -1401,36 +1415,41 @@ error:
 
 
 SIXELAPI SIXELSTATUS
-sixel_encode(unsigned char  /* in */ *pixels,     /* pixel bytes */
-             int            /* in */ width,       /* image width */
-             int            /* in */ height,      /* image height */
-             int const      /* in */ depth,       /* color depth */
-             sixel_dither_t /* in */ *dither,     /* dither context */
-             sixel_output_t /* in */ *context)    /* output context */
+sixel_encode(
+    unsigned char  /* in */ *pixels,   /* pixel bytes */
+    int            /* in */ width,     /* image width */
+    int            /* in */ height,    /* image height */
+    int const      /* in */ depth,     /* color depth */
+    sixel_dither_t /* in */ *dither,   /* dither context */
+    sixel_output_t /* in */ *output)   /* output context */
 {
     SIXELSTATUS status = SIXEL_FALSE;
 
     /* TODO: reference counting should be thread-safe */
     sixel_dither_ref(dither);
-    sixel_output_ref(context);
+    sixel_output_ref(output);
 
     (void) depth;
 
     if (dither->quality_mode == SIXEL_QUALITY_HIGHCOLOR) {
         status = sixel_encode_highcolor(pixels, width, height,
-                                        dither, context);
+                                        dither, output);
     } else {
         status = sixel_encode_dither(pixels, width, height,
-                                     dither, context);
+                                     dither, output);
     }
 
-    sixel_output_unref(context);
+    sixel_output_unref(output);
     sixel_dither_unref(dither);
 
     return status;
 }
 
-
-/* emacs, -*- Mode: C; tab-width: 4; indent-tabs-mode: nil -*- */
-/* vim: set expandtab ts=4 : */
+/* emacs Local Variables:      */
+/* emacs mode: c               */
+/* emacs tab-width: 4          */
+/* emacs indent-tabs-mode: nil */
+/* emacs c-basic-offset: 4     */
+/* emacs End:                  */
+/* vim: set expandtab ts=4 sts=4 sw=4 : */
 /* EOF */
